#define _CRT_SECURE_NO_DEPRECATE
#include "windows.h"

#pragma warning(push)
#pragma warning(disable : 4267 4244)
#include <clientapi.h>
#pragma warning( pop )

#include "fs.h"
#include "fs_client.h"
#include "fs_filesys.h"
#include "filewriter.h"

long long last_file_size = -1;


FsClientUser::FsClientUser() :
	mSyncStatCallBack(NULL),
	mFileStatCallback(NULL),
	mClobberCallback(NULL)
{
}


FileSys* FsClientUser::File(FileSysType type)
{
	Log("FsClientUser::File(%s)\n", cFileType[type & FST_MASK]);
	if ((type & FST_MASK) == FST_UNICODE)
	{
		// As we're not going to be able to track the download progress of this file,
		// just pretend we have received all of it now.
		if (FileWriter::GetReadCallback())
			FileWriter::GetReadCallback()(last_file_size);
		return FileSys::Create(type);
	}
	return new FsFileSys(type, last_file_size);
}


void FsClientUser::OutputStat(StrDict* varList)
{
	StrRef var, val;
	int file_count = -1;
	long long sync_size = -1;
	const char* depot_file = NULL;
	const char* client_file = NULL;
	int rev = -1;
	const char* action = NULL;
	last_file_size = -1;
	for (int i = 0; varList->GetVar(i, var, val); ++i)
	{
		if (::strcmp(var.Text(), "fileSize") == 0)
			last_file_size = val.Atoi64();
		else if (::strcmp(var.Text(), "totalFileSize") == 0)
			sync_size = val.Atoi64();
		else if (::strcmp(var.Text(), "totalFileCount") == 0)
			file_count = val.Atoi();
		else if (::strcmp(var.Text(), "depotFile") == 0)
			depot_file = val.Text();
		else if (::strcmp(var.Text(), "clientFile") == 0)
			client_file = val.Text();
		else if (::strcmp(var.Text(), "rev") == 0)
			rev = val.Atoi();
		else if (::strcmp(var.Text(), "action") == 0)
			action = val.Text();
	}
	if (mSyncStatCallBack && file_count >= 0)
		mSyncStatCallBack(file_count, sync_size);
	if (mFileStatCallback && last_file_size >= 0)
		mFileStatCallback(depot_file, client_file, action, rev, last_file_size);
}


void FsClientUser::HandleError(Error *inError)
{
	if (DoScriptedOutput())
		printf("error: ");
	// Collect all seen error messages so we can report them at the end.
	if (inError->GetSeverity() > E_WARN)
	{
		StrBuf msg;
		inError->Fmt(&msg);
		std::string error_msg(msg.Text());
		mErrorMessage += error_msg;
		if (mClobberCallback)
		{
			// we could also check if inError->GetGeneric() == 34, but the error codes aren't documented, 
			// and we need the filename from the error message anyway, so it seems safer to do it like this.
			const std::string clobber_message("Can't clobber writable file ");
			if (error_msg.find(clobber_message) == 0)
				mClobberCallback(std::string(error_msg, clobber_message.size(), error_msg.size() - clobber_message.size() - 1));
		}
	}
	ClientUser::HandleError(inError);
}


void FsClientUser::SetSyncStatCallback(SyncStatCallbackFunc inSyncStatCallback)
{
	mSyncStatCallBack = inSyncStatCallback;
}


void FsClientUser::SetFileStatCallback(FileStatCallbackFunc inFileStatCallback)
{
	mFileStatCallback = inFileStatCallback;
}


void FsClientUser::SetClobberCallback(ClobberCallbackFunc inClobberCallback)
{
	mClobberCallback = inClobberCallback;
}
